require 'spec_helper'

RSpec.describe Shoryuken::Worker::DefaultExecutor do
  let(:sqs_queue) { double 'SQS Queue' }
  let(:queue)     { 'default' }

  before do
    allow(Shoryuken::Client).to receive(:queues).with(queue).and_return(sqs_queue)
  end

  describe '.perform_in' do
    it 'delays a message' do
      expect(sqs_queue).to receive(:send_message).with({
                                                         message_attributes: {
                                                           'shoryuken_class' => {
                                                             string_value: TestWorker.to_s,
                                                             data_type: 'String'
                                                           }
                                                         },
                                                         message_body: 'message',
                                                         delay_seconds: 60
                                                       })

      TestWorker.perform_in(60, 'message')
    end

    it 'raises an exception' do
      expect {
        TestWorker.perform_in(901, 'message')
      }.to raise_error 'The maximum allowed delay is 15 minutes'
    end
  end

  describe '.perform_at' do
    it 'delays a message' do
      expect(sqs_queue).to receive(:send_message).with({
                                                         message_attributes: {
                                                           'shoryuken_class' => {
                                                             string_value: TestWorker.to_s,
                                                             data_type: 'String'
                                                           }
                                                         },
                                                         message_body: 'message',
                                                         delay_seconds: 60
                                                       })

      TestWorker.perform_in(Time.now + 60, 'message')
    end

    it 'raises an exception' do
      expect {
        TestWorker.perform_in(Time.now + 901, 'message')
      }.to raise_error 'The maximum allowed delay is 15 minutes'
    end
  end

  describe '.perform_async' do
    it 'enqueues a message' do
      expect(sqs_queue).to receive(:send_message).with({
                                                         message_attributes: {
                                                           'shoryuken_class' => {
                                                             string_value: TestWorker.to_s,
                                                             data_type: 'String'
                                                           }
                                                         },
                                                         message_body: 'message'
                                                       })

      TestWorker.perform_async('message')
    end

    it 'enqueues a message with options' do
      expect(sqs_queue).to receive(:send_message).with({
                                                         delay_seconds: 60,
                                                         message_attributes: {
                                                           'shoryuken_class' => {
                                                             string_value: TestWorker.to_s,
                                                             data_type: 'String'
                                                           }
                                                         },
                                                         message_body: 'delayed message'
                                                       })

      TestWorker.perform_async('delayed message', delay_seconds: 60)
    end

    it 'accepts an `queue` option' do
      new_queue = 'some_different_queue'

      expect(Shoryuken::Client).to receive(:queues).with(new_queue).and_return(sqs_queue)

      expect(sqs_queue).to receive(:send_message).with({
                                                         message_attributes: {
                                                           'shoryuken_class' => {
                                                             string_value: TestWorker.to_s,
                                                             data_type: 'String'
                                                           }
                                                         },
                                                         message_body: 'delayed message'
                                                       })

      TestWorker.perform_async('delayed message', queue: new_queue)
    end
  end
end
